/**
 * @file
 * Webform Autosave behaviors.
 */

(function($, Drupal) {
  'use strict';
  // Set our primary store.
  var store = {
    autosaveTime: parseInt(drupalSettings.webformautosave.autosave_time),
    changedTimestamp: null,
    csrfToken: null,
    optimisticLocking: drupalSettings.webformautosave.optimistic_locking === 1,
    sid: drupalSettings.webformautosave.sid,
    submissionData: {
      webform_id: drupalSettings.webformautosave.webform_id,
    },
    submissionUrl: drupalSettings.webformautosave.submission_url,
    webformId: drupalSettings.webformautosave.webform_id,
    runQuery: true,
  };

  /**
   * Getter for the CSRF Token.
   *
   * @param callback
   */
  var getCsrfToken = function(callback) {
    // Get the CSRF token if not set, then perform the callback.
    if (store.csrfToken === null) {
      $.get(Drupal.url('session/token')).done(function(data) {
        // Set the token to the returned data.
        store.csrfToken = data;
        callback(store.csrfToken);
      });
    }
    else {
      callback(store.csrfToken);
    }
  };

  /**
   * Getter for the webform submission data.
   */
  var getSubmission = function(csrfToken, updateData = {}) {
    store.runQuery = false;
    var sid = store.sid;
    var webformId = store.webformId;
    // Perform a PATCH if there is an SID.
    var getUrl =
      '/webformautosave/' + webformId + '/submission/' + sid + '?_format=json';
    $.ajax({
      url: getUrl,
      method: 'GET',
      headers: {
        'Content-Type': 'application/json',
        'X-CSRF-Token': csrfToken,
      },
    }).done(function(response) {
      // Update our store.
      store.changedTimestamp = Date.parse(response.entity.changed[0].value);
      store.submissionData = Object.assign(response.data, store.submissionData);
      store.runQuery = true;
    });
  };
  /**
   * Set a Drupal message.
   */
  var setMessage = function(message, status = 'error') {
    var error = '<div role="contentinfo" aria-label="Error message" class="messages message-body messages--' + status + '">' + message + '</div>';
    var messageWrapper = $('.messages__wrapper');
    if ($(messageWrapper).length) {
      $(messageWrapper).prepend(error);
    }
    else if ($('main').length) {
      $('main').prepend('<div class="message is-danger">' + error + '</div>');
    }
    else {
      $('.main-content').prepend('<div class="message is-danger">' + error + '</div>')
    }
    window.scrollTo(0, 0);
  };

  /**
   * Updater for the webform submission.
   */
  var updateSubmission = function(csrfToken) {
    store.runQuery = false;
    var sid = store.sid;
    var webformId = store.webformId;
    // Perform a PATCH if there is an SID.
    var patchUrl =
      '/webformautosave/' + webformId + '/submission/' + sid + '?_format=json';
    $.ajax({
      url: patchUrl,
      method: 'PATCH',
      headers: {
        'Content-Type': 'application/json',
        'X-CSRF-Token': csrfToken,
      },
      data: JSON.stringify(store.submissionData),
      error: function (request, status, error) {
        setMessage(request.responseText);
      }
    }).done(function(response) {
      // Update the current submission data.
      store.changedTimestamp = Date.parse(response.entity.changed[0].value);
      store.submissionData = Object.assign(response.data, store.submissionData);
      store.runQuery = true;
    });
  };

  /**
   * Posts a webform submission.
   */
  var postSubmission = function(csrfToken) {
    store.runQuery = false;
    // Set our webform ID.
    store.submissionData['webform_id'] = store.webformId;
    // Perform a PATCH if there is an SID.
    var postUrl = '/webformautosave/submit?_format=json';
    $.ajax({
      url: postUrl,
      method: 'POST',
      headers: {
        'Content-Type': 'application/json',
        'X-CSRF-Token': csrfToken,
      },
      data: JSON.stringify(store.submissionData),
    }).done(function(response) {
      store.sid = response.sid;
      store.changedTimestamp = Date.parse(response.entity.changed[0].value);
      store.submissionData = Object.assign(response.data, store.submissionData);
      store.runQuery = true;
    });
  };

  /**
   * Debounced action to perform on change.
   */
  var changeAction = Drupal.debounce(
    function() {
      // Patch the submission if the form has an SID.
      getCsrfToken(function(csrfToken) {
        updateSubmission(csrfToken);
      });
    },
    store.autosaveTime,
    false
  );

  /**
   * Setup our default behaviors for the webformautosave module.
   */
  Drupal.behaviors.webformautosave = {
    // eslint-disable-next-line no-unused-vars
    attach: function(context, settings) {
      // Let's bind an input event to our inputs once.
      $('form.webform-submission-form', context)
        .once('webformAutosaveBehavior')
        .each(function() {
          var webform = $(this);
          // Update our stored submission data.
          if (store.sid && store.runQuery) {
            store.runQuery = false;
            // Get the current submission data.
            getCsrfToken(function(csrfToken) {
              getSubmission(csrfToken);
            });
          }
          // Add an input event listener to each input.
          $(webform).find('input, select, textarea').change(function (event) {
            // Update the local data.
            var value = event.target.value;
            var name = event.target.name;
            store.submissionData[name] = value;
            // Only run the update if we can run a query.
            if (store.runQuery) {
              store.runQuery = false;
              changeAction(event);
            }
          });

          // Stop queries on submit.
          $(webform).submit(function () {
            store.runQuery = false;
          });
        });
    },
  };
})(jQuery, Drupal);
